//=============================================================================
// File Name: AugWeightVector.cc
// implements the methods of the AugWeightVector class
//=============================================================================
//
// Based on:
//
//  Copyright (c) 2007 Shai Shalev-Shwartz.
//  All Rights Reserved.
//=============================================================================
// File Name: WeightVector.cc
// implements the methods of the WeightVector class
//=============================================================================
//
// Distributed under GNU General Public License (see license.txt for details).

#include "AugWeightVector.h"


void AugWeightVector::add(simple_sparse_vector& x, double s) {
  double pred = 0.0, norm_x = 0.0;
  
  for( simple_sparse_vector_iterator it = x.my_vec.begin(); 
       it != x.my_vec.end(); it++) {
    double val = (*it).second * s;
    norm_x += val*val;
    pred += 2.0*my_v[(*it).first]*val;
    my_v[(*it).first] += (val/my_a);
  }
  my_snorm += norm_x + my_a*pred;
  // assume that -1 is tacked onto the end of x
  my_beta -= s/my_a;
}


// this += s*x
void AugWeightVector::add(AugWeightVector& x, double s) {
  my_snorm = 0.0;
  for (uint i=0; i<d; ++i) {
    my_v[i] *= my_a;
    my_v[i] += (x[i] * s);
    my_snorm += my_v[i]*my_v[i];
  }
  my_beta *= my_a;
  my_beta += (x.beta() * s);
  my_a = 1.0;
    
}

/*
void WeightVector::updateL1(double ustep, simple_sparse_vector& x, double s) {
  my_snorm = 0.0;
  simple_sparse_vector_iterator it = x.my_vec.begin();
  register double val;
  for (uint i=0; i<d; ++i) {
    my_v[i] *= my_a;

    if(i == (*it).first) {
      val = (*it).second * s;
      if( it != x.my_vec.end() ) it++;
    } else {
      val = 0.;
    }

    if( my_v[i] > 0 )
      my_v[i] += (-ustep + val);
    else if( my_v[i] < 0 )
      my_v[i] += (ustep + val);
    else {
      if (ustep + val < 0) my_v[i] += (ustep + val);
      else if(-ustep + val > 0) my_v[i] += (-ustep  + val);
    }

    my_snorm += my_v[i]*my_v[i];
  }
  my_a = 1.0;
}
*/

/*
void WeightVector::updateL1(double ustep) {
  my_snorm = 0.0;
  for (uint i=0; i<d; ++i) {
    my_v[i] *= my_a;
    if( my_v[i] > 0 )    my_v[i] -= ustep;
    else if(my_v[i] < 0) my_v[i] += ustep;
    my_snorm += my_v[i]*my_v[i];
  }
  my_a = 1.0;
}
*/

void AugWeightVector::print(std::ostream& os) {
  
  for(uint i=0; i < d; ++i) {
    if (my_v[i] != 0.0)
      os << i << ":" << (my_v[i]*my_a) << " "; 
  }
  os << " intercept beta:" << my_beta*my_a;
  os << std::endl;
}


//--------------------------------------------------------------------------
double operator* (simple_sparse_vector& u, AugWeightVector& v) {
  double outcome = 0.0;
  for( simple_sparse_vector_iterator it = u.my_vec.begin(); 
      it != u.my_vec.end(); it++) {
    outcome += ((*it).second * v[(*it).first]);
  }
  // assume that the intercept beta is matched with -1
  outcome -= v.beta();
  return outcome;
}

//-----------------------------------------------------------------------------
double operator* (AugWeightVector& v, simple_sparse_vector& u) {
  return (u*v);
}



