#include <stdio.h>
#include <stdlib.h>

extern "C" {
/* DSYEVX prototype */
extern void dsyevx_( char* jobz, char* range, char* uplo, int* n, double* a,
	                int* lda, double* vl, double* vu, int* il, int* iu, double* abstol,
			                int* m, double* w, double* z, int* ldz, double* work, int* lwork,
					      int* iwork, int* ifail, int* info );
/* Auxiliary routines prototypes */
extern void print_matrix( char* desc, int m, int n, double* a, int lda );
}

/* Parameters */
#define N 5
#define NSELECT 3
#define LDA N
#define LDZ N

/* Main program */
int main() {
        /* Locals */
        int n = N, il, iu, m, lda = LDA, ldz = LDZ, info, lwork;
        double abstol, vl, vu;
        //double wkopt;
	double wkopt[1];
        double* work;
        /* Local arrays */
	/* iwork dimension should be at least 5*n */
        int iwork[5*N], ifail[N];
        double w[N], z[LDZ*NSELECT];
        double a[LDA*N] = {
            6.29,  0.00,  0.00,  0.00,  0.00,
           -0.39,  7.19,  0.00,  0.00,  0.00,
            0.61,  0.81,  5.48,  0.00,  0.00,
            1.18,  1.19, -3.13,  3.79,  0.00,
           -0.08, -0.08,  0.22, -0.26,  0.83
        };
        /* Executable statements */
        printf( " DSYEVX Example Program Results\n" );
        /* Negative abstol means using the default value */
        abstol = -1.0;
        /* Set il, iu to compute NSELECT smallest eigenvalues */
        il = 1;
        iu = NSELECT;
        /* Query and allocate the optimal workspace */
        lwork = -1;
        dsyevx_( "Vectors", "Indices", "Upper", &n, a, &lda, &vl, &vu, &il, &iu,
                        &abstol, &m, w, z, &ldz, wkopt, &lwork, iwork, ifail, &info );
        //lwork = (int)wkopt;
	if (info != 0) {
		printf( "Cannot determine the optimal workspace\n");
		exit(1);
	}
	lwork = (int)wkopt[0];
	if(lwork < 8*N) lwork = 8*N;
	printf("lwork = %d\n", lwork);
        work = (double*)malloc( lwork*sizeof(double) );
        /* Solve eigenproblem */
        dsyevx_( "Vectors", "Indices", "Upper", &n, a, &lda, &vl, &vu, &il, &iu,
                        &abstol, &m, w, z, &ldz, work, &lwork, iwork, ifail, &info );
        /* Check for convergence */
        if( info > 0 ) {
                printf( "The algorithm failed to compute eigenvalues.\n" );
                exit( 1 );
        }
        /* Print the number of eigenvalues found */
        printf( "\n The total number of eigenvalues found:%2i\n", m );
        /* Print eigenvalues */
        print_matrix( "Selected eigenvalues", 1, m, w, 1 );
        /* Print eigenvectors */
        print_matrix( "Selected eigenvectors (stored columnwise)", n, m, z, ldz );
        /* Free workspace */
        free( (void*)work );
        exit( 0 );
} /* End of DSYEVX Example */

/* Auxiliary routine: printing a matrix */
void print_matrix( char* desc, int m, int n, double* a, int lda ) {
        int i, j;
        printf( "\n %s\n", desc );
        for( i = 0; i < m; i++ ) {
                for( j = 0; j < n; j++ ) printf( " %6.2f", a[i+j*lda] );
                printf( "\n" );
        }
}
