// Distributed under GNU General Public License (see license.txt for details).
//
//  Copyright (c) 2007 Shai Shalev-Shwartz.
//  All Rights Reserved.
//=============================================================================
// File Name: pegasos_optimize.h
// header for the main optimization function of pegasos
//=============================================================================

#ifndef _SHAI_PEGASOS_OPTIMIZE_H
#define _SHAI_PEGASOS_OPTIMIZE_H

//*****************************************************************************
// Included Files
//*****************************************************************************
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <sstream>
#include <ctime>
#include <cmath>
#include "simple_sparse_vec_hash.h"
#include "WeightVector.h"
#include "AugWeightVector.h"
//#include <tnt.h>
//#include <jama_eig.h>
//using namespace TNT;
//using namespace JAMA;


// function from Joachims that measures CPU time
double get_runtime(void);

// main optimization function
void Learn(// Input variables
	   std::vector<simple_sparse_vector>& Dataset,
	   std::vector<int>& Labels,
	   uint dimension,
	   std::vector<simple_sparse_vector>& testDataset,
	   std::vector<int>& testLabels,
	   double lambda,int max_iter,int exam_per_iter,int num_iter_to_avg,
	   std::string& model_filename,
	   // Output variables
	   double& train_time,double& calc_obj_time,double& obj_value,
	   double& norm_value,double& loss_value,double& zero_one_error,
	   double& test_loss,double& test_error,
	   // additional parameters
	   int eta_rule_type, double eta_constant,
	   int projection_rule, double projection_constant);

void LearnAndValidate(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      int exam_per_iter,int num_example_to_validate,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant);


void LearnReturnLast(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      int exam_per_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant);

void LearnRandomFT(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      int exam_per_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant, 
		      int rank, double gamma, int avg_no);

void LearnRobustRandomFT(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      int exam_per_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant, 
		      int rank, double gamma, int avg_no, int use_b);

void LearnNystrom(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant,
		      int sample, double gamma, double sol_eps, int avg_no);
//		      int rank, double gamma, double ignore_ratio);

void LearnNystromRobust(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double theta,
		      int projection_rule, double projection_constant,
		      int sample, double gamma, double sol_eps, int avg_no);

void LearnNystromLowdim(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int eta_rule_type , double eta_constant ,
		      int projection_rule, double projection_constant,
		      int sample, double gamma, double sol_eps, int avg_no);

void LearnNystromRobustLowdim(// Input variables
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      double lambda,int max_iter,
		      std::string& model_filename,
		      // Output variables
		      double& train_time,double& calc_obj_time,
		      double& obj_value,double& norm_value,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      // additional parameters
		      int steplength_rule, double theta,
		      int projection_rule, double projection_constant,
		      int sample, double gamma, double sol_eps, int avg_no, int use_b);

void compute_error_random(// Input variables
		      WeightVector& alp,
		      std::vector<simple_sparse_vector>& Dataset,
		      std::vector<int>& Labels,
		      uint dimension,
		      std::vector<simple_sparse_vector>& testDataset,
		      std::vector<int>& testLabels,
		      // Output variables
		      double& precompute_time, double& calc_train_time, double& calc_test_time,
		      double& loss_value,double& zero_one_error,
		      double& test_loss,double& test_error,
		      int rank, double gamma, double ignore_ratio,
		      double proj_const);

void LearnRobust(// Input variables
		 std::vector<simple_sparse_vector>& Dataset,
		 std::vector<int>& Labels,
		 uint dimension,
		 std::vector<simple_sparse_vector>& testDataset,
		 std::vector<int>& testLabels,
		 double lambda,int max_iter,
		 int exam_per_iter,
		 double frac_iter_to_avg,
		 std::string& model_filename,
		 // Output variables
		 double& train_time,double& calc_obj_time,
		 double& obj_value,double& norm_value,
		 double& loss_value,double& zero_one_error,
		 double& test_loss,double& test_error,
		 // additional parameters for steplength
		 int eta_rule_type , double theta,
		 int projection_rule, double projection_constant,
		 int n_grad_average);

void LearnRobustAug(// Input variables
		 std::vector<simple_sparse_vector>& Dataset,
		 std::vector<int>& Labels,
		 uint dimension,
		 std::vector<simple_sparse_vector>& testDataset,
		 std::vector<int>& testLabels,
		 double lambda,int max_iter,
		 int exam_per_iter,
		 double frac_iter_to_avg,
		 std::string& model_filename,
		 // Output variables
		 double& train_time,double& calc_obj_time,
		 double& obj_value,double& norm_value,
		 double& loss_value,double& zero_one_error,
		 double& test_loss,double& test_error,
		 // additional parameters for steplength
		 int eta_rule_type , double theta,
		 int projection_rule, double projection_constant,
		 int n_grad_average);

// function for reading the data
void ReadData(// input
	      std::string& data_filename,
	      // output
	      std::vector<simple_sparse_vector> & Dataset,
	      std::vector<int> & Labels,
	      uint& dimension,
	      double& sparsity,
	      double& readingTime);


// function for running experiments
void run_experiments(std::string& experiments_file,
		     std::vector<simple_sparse_vector>& Dataset,
		     std::vector<int>& Labels,
		     uint dimension,
		     std::vector<simple_sparse_vector>& testDataset,
		     std::vector<int>& testLabels);

#define MAX(a,b) ((a)>(b) ? (a):(b))
#define MIN(a,b) ((a)>(b) ? (b):(a))

#define SWAP(a,b,type) { type tmp; tmp = a; a = b; b = tmp; }

#endif
